(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - MODUL  DEPT                                      02.11.93  *)
(*                                                                          *)
(*  (c) Markus Neher    Institute for Applied Mathematics                   *)
(*                      D-76128 Karlsruhe University, Germany               *)
(*                      e-mail: ae16@rz.uni-karlsruhe.de                    *)
(*                                                                          *)
(*  BERECHNUNG VON NHERUNGEN UND EINSCHLIESSUNGEN VON EIGENWERTEN UND      *)
(*  EIGENFUNKTIONEN DER STURM-LIOUVILLE-GLEICHUNG ( DIREKTES PROBLEM )      *)
(*                                                                          *)
(*                  - U" + P(X)  U(X) =   U(X)                           *)
(*                         U(0) = U() = 0                                  *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

MODULE dept;

USE i_ari , mv_ari , mvi_ari;
USE iep_rawa , iep_iawa , iep_tayk , seig_app , service;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR DEPT_RITZ                               13.03.93  *)
(*                                                                          *)
(*  BERECHNET FR EINE ENDLICHE COSINUSREIHE NHERUNGEN FR DIE ERSTEN N+1  *)
(*  EIGENWERTE MIT DEM RITZ-VERFAHREN ZU DEN ANSATZFUNKTIONEN SIN(kx)       *)
(*  SOWIE DIE FUNKTIONALMATRIX BEIM NEWTON-VERFAHREN FR DAS INVERSE        *)
(*  DIRICHLETPROBLEM AUS DER DISSERTATION                                   *)
(*                                                                          *)
(*  DIE KOEFFIZIENTEN DER COSINUSREIHE STEHEN IN FCP_AKT                    *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE DEPT_RITZ( FCP_AKT : RVECTOR; N , ANZ : INTEGER;
                            VAR EIGVAL : RVECTOR; VAR FUNKT_MAT : RMATRIX );

   (*  AUSGEGEBEN WERDEN DIE KLEINSTEN N+1 EIGENWERTE IN EIGVAL UND DIE     *)
   (*  FUNKTIONALMATRIX BEIM NEWTON-VERFAHREN ZUM INVERSEN DP IN FUNKT_MAT  *)
   
   (*  N   : N + 1 = ANZAHL DER VORGEGEBENEN EIGENWERTE VON P(X)            *)

   (*  ANZ = ANZAHL DER ANSATZFUNKTIONEN BEIM RITZ-VERFAHREN                *)
   (*        (JE FR GERADE UND UNGERADE EIGENWERTE)                        *)

CONST PI = 3.141592653589793;

TYPE MATSARRAY = DYNAMIC ARRAY[*,*,*] OF REAL;

VAR I , J , K             : INTEGER;
    H1                    : REAL;
    VALID                 : BOOLEAN;
    RITZEIGVAL , VR1      : RVECTOR[1..ANZ];
    RITZMAT , RITZEIGVEC  : RMATRIX[1..ANZ,1..ANZ];
    BASIS_RITZMATS        : MATSARRAY[0..N,1..ANZ,1..ANZ];
    FILENAMEG , FILENAMEU : STRING;
    FILEG , FILEU         : TEXT;

(*--------------------------------------------------------------------------*)

BEGIN

   (*  ERSTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  *)
   (*  MIT GERADEM INDEX :  BASIS-RITZ-MATRIX BESETZEN  *)

   BASIS_RITZMATS[0,*,*] := ID( ANZ );

   FOR K := 1 TO N DO BEGIN

      BASIS_RITZMATS[K,*,*] := NULL( ANZ );

      FOR I := K TO K DO
          FOR J := I + 1 TO ANZ DO BEGIN
              BASIS_RITZMATS[K,J,J-I] := 0.5;
              BASIS_RITZMATS[K,J-I,J] := 0.5;
          END;

      FOR I := K TO K DO
          FOR J := 1 TO I DO
              BASIS_RITZMATS[K,I+1-J,J] := BASIS_RITZMATS[K,I+1-J,J] - 0.5;

   END;

   (*  ZWEITER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  *)
   (*  MIT GERADEM INDEX :  RITZ-MATRIX BESETZEN         *)

   RITZMAT := FCP_AKT[0] * BASIS_RITZMATS[0,*,*];

   FOR I := 1 TO N DO
       RITZMAT := RITZMAT + FCP_AKT[I] * BASIS_RITZMATS[I,*,*];

   (*  DIAGONALTERM  " - U_J'' U_I "  ADDIEREN  *)

   FOR I := 1 TO 30 DO
       RITZMAT[I,I] := RITZMAT[I,I] + SQR( 2 * I - 1 );

   (*  DRITTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  MIT             *)
   (*  GERADEM INDEX :  MODUL SEIG_APP ZUR EIGENWERTBERECHNUNG AUFRUFEN  *)

   SEIG_APP( RITZMAT , RITZEIGVAL , RITZEIGVEC );

   (*  KLEINSTE  N DIV 2 + 1  EIGENWERTE SORTIEREN  *)

   K := 0;
   REPEAT VALID := TRUE;
          K := K + 1;
          FOR I := ANZ DOWNTO K + 1 DO
              IF RITZEIGVAL[I] < RITZEIGVAL[I-1] THEN BEGIN
                 VALID := FALSE;
                 H1 := RITZEIGVAL[I];
                 VR1 := RITZEIGVEC[*,I];
                 RITZEIGVAL[I] := RITZEIGVAL[I-1];
                 RITZEIGVEC[*,I] := RITZEIGVEC[*,I-1];
                 RITZEIGVAL[I-1] := H1;
                 RITZEIGVEC[*,I-1] := VR1;
              END;
   UNTIL  VALID OR ( K = N DIV 2 + 1 );

   FOR I := 0 TO N DIV 2 DO
       EIGVAL[2*I] := RITZEIGVAL[I+1];

   (*  FUNKTIONALMATRIX BERECHNEN  *)

   FOR I := 0 TO N DIV 2 DO
       FOR J := 0 TO N DO
           FUNKT_MAT[2*I,J] := RITZEIGVEC[*,I+1] *
                               ( BASIS_RITZMATS[J,*,*] * RITZEIGVEC[*,I+1] );

   (*  ERSTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)    *)
   (*  MIT UNGERADEM INDEX :  BASIS-RITZ-MATRIX BESETZEN  *)

   BASIS_RITZMATS[0,*,*] := ID( ANZ );

   FOR K := 1 TO N DO BEGIN

      BASIS_RITZMATS[K,*,*] := NULL( ANZ );

      FOR I := K TO K DO
          FOR J := I + 1 TO ANZ DO BEGIN
              BASIS_RITZMATS[K,J,J-I] := 0.5;
              BASIS_RITZMATS[K,J-I,J] := 0.5;
          END;

      FOR I := K TO K DO
          FOR J := 1 TO I - 1 DO
              BASIS_RITZMATS[K,I-J,J] := BASIS_RITZMATS[K,I-J,J] - 0.5;

   END;

   (*  ZWEITER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  *)
   (*  MIT UNGERADEM INDEX :  RITZ-MATRIX BESETZEN       *)

   RITZMAT := FCP_AKT[0] * BASIS_RITZMATS[0,*,*];

   FOR I := 1 TO N DO
       RITZMAT := RITZMAT + FCP_AKT[I] * BASIS_RITZMATS[I,*,*];

   (*  DIAGONALTERM  " - U_J'' U_I "  ADDIEREN  *)

   FOR I := 1 TO 30 DO
       RITZMAT[I,I] := RITZMAT[I,I] + SQR( 2 * I );

   (*  DRITTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  MIT               *)
   (*  UNGERADEM INDEX :  MODUL SEIG_APP ZUR EIGENWERTBERECHNUNG AUFRUFEN  *)

   SEIG_APP( RITZMAT , RITZEIGVAL , RITZEIGVEC );

   (*  KLEINSTE  ( N + 1 ) DIV 2  EIGENWERTE SORTIEREN  *)

   K := 0;
   REPEAT VALID := TRUE;
          K := K + 1;
          FOR I := ANZ DOWNTO K + 1 DO
              IF RITZEIGVAL[I] < RITZEIGVAL[I-1] THEN BEGIN
                 VALID := FALSE;
                 H1 := RITZEIGVAL[I];
                 VR1 := RITZEIGVEC[*,I];
                 RITZEIGVAL[I] := RITZEIGVAL[I-1];
                 RITZEIGVEC[*,I] := RITZEIGVEC[*,I-1];
                 RITZEIGVAL[I-1] := H1;
                 RITZEIGVEC[*,I-1] := VR1;
              END;
   UNTIL  VALID OR ( K = ( N + 1 ) DIV 2 );

   FOR I := 1 TO ( N + 1 ) DIV 2 DO
       EIGVAL[2*I-1] := RITZEIGVAL[I];

   (*  FUNKTIONALMATRIX BERECHNEN  *)

   FOR I := 1 TO ( N + 1 ) DIV 2 DO
       FOR J := 0 TO N DO
           FUNKT_MAT[2*I-1,J] := RITZEIGVEC[*,I] *
                                 ( BASIS_RITZMATS[J,*,*] * RITZEIGVEC[*,I] );

END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR DEPT_RITZ                                         *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)



(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EW_APP                                  02.02.93  *)
(*                                                                          *)
(*  BERECHNET FR DIE FUNKTION TRVECP NHERUNGEN FR EINEN EIGENWERT        *)
(*  IN DER NHE DES EINGABEPARAMETERS EIG                                   *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME MIT AWA GELST                                      *)
(*  ETWAS GENAUER, ABER VOR ALLEM LANGSAMER ALS DAS RITZ-VERFAHREN          *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

GLOBAL FUNCTION EW_APP( M , P , EW , DIVI : INTEGER; TRVECP : RTVEC;
       EIG : REAL; VAR AUS : TEXT ) : REAL;

       (*  P  :  ORDNUNG DER TAYLORENTWICKLUNG VON U  *)
       (*  M  :  SCHRITTWEITE  H = PI / M             *)
       (*  EW :  NUMMER DES EIGENWERTS                *)
       (*  TRVECP ENTHLT DIE TAYLORKOEFFIZIENTEN DER FUNKTION  *)
       (*  IN DEN STTZSTELLEN (GEEIGNET NORMIERT)              *)


VAR DIVI_2 , COUNT             : INTEGER;
    EIGALT , STEIGUNG , PI , H : REAL;
    VRU , VRV                  : RVECTOR[1..2];

(*SUB-*)
PROCEDURE DRUCK_AWA; (*-----------------------------------------------------*)
  
(*  DRUCKT ERGEBNISSE DER ANFANGSWERTPROBLEME  *)

VAR J : INTEGER;

BEGIN
   WRITELN( AUS,'Funktionswerte von u(x) am rechten Rand fr  =',EIG,' :');
   WRITELN( AUS );
   WRITELN( AUS , 'U (' , DIVI : 1 , ') = ' , VRU[1] );
   WRITELN( AUS , 'U''(' , DIVI : 1 , ') = ' , VRU[2] );
   WRITELN( AUS );
END;         

(*--- HAUPTPROGRAMM EW_APP -------------------------------------------------*)

BEGIN 

   (*  FR DEN EIGENWERT EW WIRD MIT DEM SEKANTEN-  *)
   (*  VERFAHREN EINE GUTE NHERUNG BERECHNET       *)

   (* GLOBALE SCHRITTWEITE SETZEN *)

   PI:= 3.141592653589793;
   H := PI / M;
   DIVI_2 := SQR( DIVI );

   COUNT := 0;
   WRITE(   'Startnherung des ' , EW : 1 , '. Eigenwerts : ' );
   WRITELN( EIG );
   WRITELN( AUS );
   WRITE(   AUS, '---  Startnherung des ' ,  EW : 1 , '. Eigenwerts : ' );
   WRITELN( AUS , EIG , '  --------------' );
   WRITELN( AUS );
   WRITELN( AUS );
    
   (*  ZUM EINSTIEG IN DAS SEKANTENVEFAHREN                         *)
   (*  FUNKTIONSWERTE IN DER NHE VON EIG BERECHNEN                 *)
   (*  EIGALT ENTHLT DEN WERT DES EIGENWERTS IM "LETZTEN" SCHRITT  *)

(* EIGALT := 1.001 * EIG; *)
   EIGALT := 1.000001 * EIG; 

   (*  ZUM START DES SEKANTENVERFAHRENS WIRD DER FUNKTIONSWERT  *)
   (*  AN DER STELLE EIGALT (IN DER NHE VON EIG BERECHNET      *)

   VRU := AWA( M * DIVI , P , H , EIGALT / DIVI_2 , TRVECP );
    
   (* VRV ENTHLT DEN "FUNKTIONSWERT" DES "LETZTEN" SCHRITTS *)

   VRV := VRU;

   (* JETZT STARTET DAS SEKANTENVERFAHREN *)
         
   REPEAT

      COUNT := COUNT + 1;

      VRU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TRVECP );
      DRUCK_AWA;

      STEIGUNG := ( VRU[1] - VRV[1] ) / ( EIG - EIGALT );
      EIGALT := EIG;
      IF STEIGUNG <> 0 THEN
         EIG := EIG - VRU[1] / STEIGUNG;
      VRV := VRU;
      
      WRITELN( COUNT : 1 , '. Nherung : ' , EIG );
      WRITE(   AUS, COUNT : 1 , '. Nherung des ' );
      WRITELN( AUS , EW : 1 , '. Eigenwerts : ' , EIG );
      WRITELN( AUS );
    
   UNTIL ( ( ABS( EIG - EIGALT ) < 3.0E-12 * ABS( EIGALT ) )
         OR ( STEIGUNG = 0 ) );

   VRU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TRVECP );
   DRUCK_AWA;  

   EW_APP := EIG;

END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EW_APP                                            *)
(*                                                                          *)
(*  BERECHNET FR DIE FUNKTION TRVECP NHERUNGEN FR EINEN EIGENWERT        *)
(*  IN DER NHE DES EINGABEPARAMETERS EIG                                   *)
(*                                                                          *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME MIT AWA GELST                                      *)
(*  ETWAS GENAUER, ABER VOR ALLEM LANGSAMER ALS DAS RITZ-VERFAHREN          *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

                                                                           

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EW_BDS                                  12.02.93  *)
(*                                                                          *)
(*  BERECHNET FR DIE FUNKTION TRVECP SCHRANKEN FR EINEN EIGENWERT         *)
(*  IN DER NHE DES EINGABEPARAMETERS EIG                                   *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME MIT AWA GELST                                      *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

GLOBAL FUNCTION EW_BDS( M, P, EW, DIVI : INTEGER; TIVECP, TIVECP_RST : ITVEC;
                EIG_APP : INTERVAL; PREC : REAL; VAR AUS : TEXT ) : INTERVAL;

       (*  P  :  ORDNUNG DER TAYLORENTWICKLUNG VON U  *)
       (*  M  :  SCHRITTWEITE  H = PI / M             *)
       (*  EW :  NUMMER DES EIGENWERTS                *)
       (*  PREC : GENAUIGKEITSANFORDERUNGEN IN %      *)
       (*  TIVECP ENTHLT DIE TAYLORKOEFFIZIENTEN DER FUNKTION  *)
       (*  IN DEN STTZSTELLEN (GEEIGNET NORMIERT)              *)


VAR COUNT , I                           : INTEGER;
    EIG , STEIGUNG , INCREASE           : REAL;
    EIGLOW , EIGHIGH , FCULOW , FCUHIGH : INTERVAL;
    PI , H , DIVI_2 , EIGBDS            : INTERVAL;
    VIU                                 : IVECTOR[1..2];

(*SUB-*)
PROCEDURE DRUCK( C1 , C2 : CHAR );  (*--------------------------------------*)

(*  HUFIG VERWENDETER STANDARD-AUSDRUCK VON ERGEBNISSEN  *)

BEGIN WRITE( EIG );
   WRITE( AUS , '>' , EIG  );
   IF C1 = 'k' THEN BEGIN
      WRITE( '  ist keine garantierte ' );
      WRITE( AUS , '  ist keine garantierte ' );
      END
   ELSE BEGIN
      WRITE( '  ist eine garantierte ' );
      WRITE( AUS , '  ist eine garantierte ' );
   END;
   IF C2 = 'o' THEN BEGIN
      WRITELN( 'obere Schranke des ' , EW : 1 , '. EW' );
      WRITELN( AUS , 'obere Schranke des ' , EW : 1 , '. EW' );
      END
   ELSE BEGIN
      WRITELN( 'untere Schranke des ' , EW : 1 , '. EW' );
      WRITELN( AUS , 'untere Schranke des ' , EW : 1 , '. EW' );
   END;
   WRITELN( AUS );

END;

(*SUB-*)
PROCEDURE DRUCK_BDS( I : INTEGER );  (*-------------------------------------*)

BEGIN

   WRITELN;
   IF I = 1 THEN BEGIN
      WRITE( 'Grobeinschlieung der Eigenwertschranken nach ' );
      WRITELN( COUNT : 1 , ' Funktionsauswertungen :' );
      END
   ELSE
      WRITELN( COUNT : 1 , '. Einschlieung der Eigenwertschranken :' );
   WRITELN;
   IWRITELN( EIGLOW ); 
   IWRITELN( EIGHIGH );
   WRITELN;

   IF I = 1 THEN BEGIN
      WRITE(   AUS , 'Grobeinschlieung der Eigenwertschranken nach ' );
      WRITELN( AUS , COUNT : 1 , ' Funktionsauswertungen :' );
      END
   ELSE
      WRITELN( AUS, COUNT : 1 , '. Einschlieung der Eigenwertschranken :' );
   WRITELN( AUS );
   IWRITELN( AUS , EIGLOW );
   IWRITELN( AUS , EIGHIGH );
   WRITELN( AUS );

END;

(*SUB-*)
PROCEDURE DRUCK_AWA;  (*----------------------------------------------------*)
  
(*  DRUCKT ERGEBNISSE DER ANFANGSWERTPROBLEME  *)

VAR J : INTEGER;

BEGIN
   WRITELN( AUS,'Funktionswerte von u(x) am rechten Rand fr  =',EIG,' :');
   WRITELN( AUS );
   WRITE( AUS , 'U (' , DIVI : 1 , ') = ' );
   IWRITELN( AUS , VIU[1] );
   WRITE( AUS , 'U''(' , DIVI : 1 , ') = ' );
   IWRITELN( AUS , VIU[2] );
   WRITELN( AUS );
END;         

(*SUB-*)
PROCEDURE IST_USCHRANKE;  (*------------------------------------------------*)
 
(*  DIE AKTUELLE NHERUNG IST EINE GARANTIERTE UNTERSCHRANKE  *) 
(*  DES EIGENWERTS                                            *)
          
BEGIN DRUCK( ' ' , 'u' );

   EIGLOW.INF := EIG;
   FCULOW.INF := VIU[1].INF;
          
END;      
 
(*SUB-*)
PROCEDURE KEINE_USCHRANKE;  (*----------------------------------------------*)

(*  DIE AKTUELLE NHERUNG IST KEINE GARANTIERTE *)
(*  UNTERSCHRANKE DES EIGENWERTS MEHR           *)

BEGIN DRUCK( 'k' , 'u' );

   EIGLOW.SUP := EIG;
   FCULOW.SUP := VIU[1].INF;
   
END;
 
(*SUB-*)
PROCEDURE IST_OSCHRANKE;  (*------------------------------------------------*)

BEGIN DRUCK( ' ' , 'o' );

   EIGHIGH.SUP := EIG;
   FCUHIGH.SUP := VIU[1].SUP;

END;

(*SUB-*)
PROCEDURE KEINE_OSCHRANKE;  (*----------------------------------------------*)

BEGIN DRUCK( 'k' , 'o' );

   EIGHIGH.INF := EIG;
   FCUHIGH.INF := VIU[1].SUP;
                
END;

(*SUB-*)
FUNCTION VALID : BOOLEAN;  (*-----------------------------------------------*)
 
(*  ABBRUCHBEDINGUNG BEI DER ITERATIONSSCHLEIFE  *)
 
BEGIN VALID := TRUE;

   (*  ITERATION BIS ZUR FAST MAXIMAL GENAUEN EINSCHLIESSUNG  *)
   (*  ABBRUCH, FALLS MAXIMAL GENAUE EINSCHLIESSUNG ODER      *)
   (*  FALLS DIE "USSERE" EINSCHLIESSUNG DIE "INNERE" NUR    *)
   (*  GERINGFGIG BERSCHTZT (PARAMETER PREC!)              *)

   IF ( ( PREC > 0 ) AND ( PREC < 1 ) AND
      ( EIGHIGH.INF -> EIGLOW.SUP > PREC *< ( EIGHIGH.SUP -< EIGLOW.INF ) ) )
      THEN VALID := FALSE;

   (*  BEI GANZ ENGER EINSCHLIESSUNG ZUSTZLICHES KRITERIUM NTIG  *)

   IF ( EIGLOW.INF  +> 1.0E-99 >= EIGLOW.SUP  -< 1.0E-99 ) AND
      ( EIGHIGH.INF +> 1.0E-99 >= EIGHIGH.SUP -< 1.0E-99 ) THEN
      VALID := FALSE;
    
END;

(*--- HAUPTPROGRAMM EW_BDS -------------------------------------------------*)

BEGIN

   (*  FR DEN EIGENWERT WERDEN MIT DEM BISEKTIONSVERFAHREN     *)
   (*  ERST EINE UNTER-, DANN EINE OBERSCHRANKE BERECHNET       *)

   (*  DA DIE "FUNKTIONSWERTE" DER "EIGENFUNKTIONEN" WEGEN DEN  *)
   (*  AUFTRETENDEN RUNDUNGSFEHLERN UNSTETIG VON DEN EIGENWERT- *)
   (*  NHERUNGEN ABHNGEN, IST DAS SEKANTENVERFAHREN HIER      *)
   (*  NICHT BRAUCHBAR --> REGULA FALSI                         *)

   (*  ZUR BERECHNUNG EINER GROBEINSCHLIESSUNG IST JEWEILS      *)
   (*  EINE VORITERATION IN DER NHE DES EIGENWERTS NTIG       *)
   (*  ES WIRD VORAUSGESETZT, DASS EINE GUTE NHERUNG DES       *)  
   (*  EIGENWERTS EINGELESEN WIRD                               *)

   (*  GLOBALE SCHRITTWEITE SETZEN  *)

   PI:= INTVAL( RVAL('3.141592653589793',-1) , RVAL('3.141592653589794',1) );
   H := PI / M;
   DIVI_2 := SQR( DIVI );
   EIG := MID( EIG_APP );
   FCULOW := 0;
   FCUHIGH := 0;

   COUNT := 1;
   WRITE(   'Startnherung des ' , EW : 1 , '. Eigenwerts : ' );
   WRITELN( EIG );
   WRITELN( AUS );
   WRITE(   AUS, '---  Startnherung des ' ,  EW : 1 , '. Eigenwerts : ' );
   WRITELN( AUS , EIG , '  --------------' );
   WRITELN( AUS );
   WRITELN( AUS );
    
   (*  SCHRITTWEITE BEI DER VORITERATION MINDESTENS     *)
   (*  HALBER DURCHMESSER DER EIGENWERTANFANGSNHERUNG  *)
   
   INCREASE := ( 1 + ABS( EIG ) ) * 2.0E-14;
   INCREASE := INCREASE + ( EIG_APP.SUP - EIG_APP.INF ) / 2;

   WRITE(   AUS, 'Mindestschrittweite bei der Voriteration : ' );
   WRITELN( AUS , INCREASE );
   WRITELN( AUS );

   (*  ZUM EINSTIEG IN DAS BISEKTIONSVEFAHREN  *)
   (*  FUNKTIONSWERTE FR EIG BERECHNEN        *)

   VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
   VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
   DRUCK_AWA;

   (*  ZUERST GROBEINSCHLIESSUNG DER UNTERSCHRANKE BERECHNEN *)
     
   IF VIU[1].INF >= 0 THEN BEGIN
    
      IST_USCHRANKE;
      KEINE_OSCHRANKE;
      
      (*  JETZT IST EINE NICHT MEHR GARANTIERBARE (ZU GROSSE)  *)
      (*  UNTERSCHRANKE ZU BERECHNEN                           *)
       
      REPEAT
        
         (*  HEURISTISCHE ERHHUNG UM DEN DURCHMESSER DER FUNKTION  *)
         (*  UND DEN WERT ABS( U() ) / ABS( VIU[2].SUP )           *)
         
         COUNT := COUNT + 1;

         EIG := EIG + INCREASE + VIU[1].INF / ABS( VIU[2].SUP );
         INCREASE := 2 * INCREASE;

         VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].INF >= 0 THEN BEGIN
            IST_USCHRANKE;
            KEINE_OSCHRANKE;
            END
         ELSE BEGIN
            KEINE_USCHRANKE;
            IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE
            ELSE KEINE_OSCHRANKE;
            END;

      UNTIL VIU[1].INF < 0;
       
      END

   ELSE BEGIN

      KEINE_USCHRANKE;
      IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE
      ELSE KEINE_OSCHRANKE;
       
      REPEAT
        
         (*  HEURISTISCHE VERMINDERUNG UM DEN DURCHMESSER DER FUNKTION  *)
         (*  UND DEN WERT ABS( U() ) / ABS( VIU[2].SUP )               *)
         
         COUNT := COUNT + 1;
                                 
         EIG := EIG - INCREASE + VIU[1].INF / ABS( VIU[2].SUP );
         INCREASE := 2 * INCREASE;

         VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].INF >= 0 THEN BEGIN
            IST_USCHRANKE;
            IF FCUHIGH.INF = 0 THEN KEINE_OSCHRANKE;(* -> NOCH KEINE UNTER-*)
            END                                       (* SCHRANKE DER OBER-*)
         ELSE BEGIN                                   (* SCHRANKE GEFUNDEN *)
            KEINE_USCHRANKE;
            IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE
            ELSE IF FCUHIGH.INF = 0 THEN KEINE_OSCHRANKE;  (* -^  *)
            END;

      UNTIL VIU[1].INF >= 0;

   END;

   (*  EVENTUELL MUSS NOCH EINE GARANTIERTE  *)
   (*  OBERSCHRANKE BERECHNET WERDEN         *)

   IF FCUHIGH.SUP = 0 THEN BEGIN

      (*  STARTWERTE VORBESETZEN *)

      INCREASE := ( 1 + ABS( EIG ) ) * 2.0E-14;
      INCREASE := INCREASE + ( EIG_APP.SUP - EIG_APP.INF ) / 2;

      EIG := EIGHIGH.INF;
      VIU[1].SUP := FCUHIGH.INF;
      VIU[2].SUP := 1.0E+14 * VIU[1].SUP;  (*  DUMMY-WERT  *)

      (*  JETZT IST EINE GARANTIERTE OBERSCHRANKE ZU BERECHNEN  *)

      REPEAT
        
         (*  HEURISTISCHE ERHHUNG UM DEN DURCHMESSER DER FUNKTION  *)
         (*  UND DEN WERT ABS( U() ) / ABS( VIU[2].SUP )           *)
         
         COUNT := COUNT + 1;

         EIG := EIG + INCREASE + VIU[1].SUP / ABS( VIU[2].SUP );
         INCREASE := 2 * INCREASE;

         VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE

         ELSE KEINE_OSCHRANKE;

      UNTIL VIU[1].SUP <= 0;

   END;

   DRUCK_BDS( 1 );

   (*  SOLANGE DIE EINSCHLIESSUNGEN DER UNTER- UND DER OBERSCHRANKE        *)
   (*  BEREINSTIMMEN, KANN SIMULTAN MIT DER REGULA FALSI ITERIERT WERDEN  *)

   WHILE EIGHIGH = EIGLOW DO BEGIN

         COUNT := COUNT + 1;

         STEIGUNG:= ( FCUHIGH.SUP-FCULOW.INF ) / ( EIGHIGH.SUP-EIGLOW.INF );
         (*  STEIGUNG IST <> 0 , WEIL FCUHIGH.SUP * FCULOW.INF <> 0  *)

         EIG := EIGLOW.INF - FCULOW.INF / STEIGUNG;

         (*  HEURISTISCHE KORREKTUR BEI ALLZU UNSYMMETRISCHER TEILUNG  *)

         IF ( EIG - EIGLOW.INF ) < ( EIGHIGH.SUP - EIGLOW.INF ) / 20 THEN
            EIG := EIGLOW.INF -> 2 *> FCULOW.INF /> STEIGUNG;

         IF ( EIGHIGH.SUP - EIG ) < ( EIGHIGH.SUP - EIGLOW.INF ) / 20 THEN
            EIG := 2 * EIG -< EIGHIGH.SUP;

         VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].INF >= 0 THEN BEGIN
            IST_USCHRANKE;
            KEINE_OSCHRANKE;
            END
         ELSE IF VIU[1].SUP <= 0 THEN BEGIN
            IST_OSCHRANKE;
            KEINE_USCHRANKE;
            END
         ELSE BEGIN
            KEINE_USCHRANKE;
            KEINE_OSCHRANKE;
            END;

         DRUCK_BDS( 0 );

   END;  (* WHILE *)

   DRUCK_BDS( 1 );
   
   (*  DIE GROBEINSCHLIESSUNGEN EIGLOW UND EIGHIGH SIND JETZT  *)
   (*  MIT DER REGULA FALSI SEPARAT ZU VERBESSERN              *)

   COUNT := 1;
    
   WHILE VALID DO BEGIN
      
      IF EIGLOW.SUP - EIGLOW.INF >= EIGHIGH.SUP - EIGHIGH.INF THEN BEGIN

         (*  UNTERSCHRANKE VERBESSERN                             *)
         (*  IN DER BEDINGUNG STEHT  ">=" , DAMIT BEI GLEICHHEIT  *)
         (*  KEINE ENDLOSSCHLEIFE DURCHLAUFEN WIRD                *)

         COUNT := COUNT + 1;

         STEIGUNG:= ( FCULOW.SUP-FCULOW.INF ) / ( EIGLOW.SUP-EIGLOW.INF );
         (*  STEIGUNG IST <> 0 , WEIL FCULOW.SUP * FCULOW.INF <> 0  *)

         EIG := EIGLOW.INF - FCULOW.INF / STEIGUNG;

         (*  HEURISTISCHE KORREKTUR BEI ALLZU UNSYMMETRISCHER TEILUNG  *)

         IF ( EIG - EIGLOW.INF ) < ( EIGLOW.SUP - EIGLOW.INF ) / 20 THEN
            EIG := EIGLOW.INF - 2 * FCULOW.INF / STEIGUNG;

         IF ( EIGLOW.SUP - EIG ) < ( EIGLOW.SUP - EIGLOW.INF ) / 20 THEN
            EIG := 2 * EIG - EIGLOW.SUP;

         IF EIG <= EIGLOW.INF THEN
            EIG := EIGLOW.INF +> ( EIGLOW.SUP - EIGLOW.INF ) /> 4;

         IF EIG >= EIGLOW.SUP THEN
            EIG := EIGLOW.SUP -< ( EIGLOW.SUP - EIGLOW.INF ) /> 4;

         VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].INF >= 0 THEN IST_USCHRANKE

         ELSE KEINE_USCHRANKE;

         DRUCK_BDS( 0 );

      END; (* IF *)
       
      IF ( VALID AND (EIGHIGH.SUP-EIGHIGH.INF >= EIGLOW.SUP-EIGLOW.INF) )
         THEN BEGIN

         (*  OBERSCHRANKE VERBESSERN  *)

         COUNT := COUNT + 1;

         STEIGUNG:= (FCUHIGH.SUP-FCUHIGH.INF) / (EIGHIGH.SUP-EIGHIGH.INF);
         (*  STEIGUNG IST <> 0 , WEIL FCUHIGH.SUP * FCUHIGH.INF <> 0  *)

         EIG := EIGHIGH.INF - FCUHIGH.INF / STEIGUNG;

         (*  HEURISTISCHE KORREKTUR BEI ALLZU UNSYMMETRISCHER TEILUNG  *)

         IF ( EIG - EIGHIGH.INF ) < ( EIGHIGH.SUP - EIGHIGH.INF )/ 20 THEN
            EIG := EIGHIGH.INF - 2 * FCUHIGH.INF / STEIGUNG;

         IF ( EIGHIGH.SUP - EIG ) < ( EIGHIGH.SUP - EIGHIGH.INF )/ 20 THEN
            EIG := 2 * EIG - EIGHIGH.SUP;

         IF EIG <= EIGHIGH.INF THEN
            EIG := EIGHIGH.INF +> ( EIGHIGH.SUP - EIGHIGH.INF ) /> 4;

         IF EIG >= EIGHIGH.SUP THEN
            EIG := EIGHIGH.SUP -< ( EIGHIGH.SUP - EIGHIGH.INF ) /> 4;

         VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE

         ELSE KEINE_OSCHRANKE;

         DRUCK_BDS( 0 );

      END; (* IF *)

   END; (* WHILE *)
       
   EIGBDS.INF := EIGLOW.INF;
   EIGBDS.SUP := EIGHIGH.SUP;

   EW_BDS := EIGBDS;

END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EW_BDS                                            *)
(*                                                                          *)
(*  BERECHNET FR DIE FUNKTION TRVECP SCHRANKEN FR EINEN EIGENWERT         *)
(*  IN DER NHE DES EINGABEPARAMETERS EIG                                   *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME MIT AWA GELST                                      *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

END.
                                                                           
(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - MODUL  DEPT                                                *)
(*                                                                          *)
(*  BERECHNUNG VON NHERUNGEN UND EINSCHLIESSUNGEN VON EIGENWERTEN UND      *)
(*  EIGENFUNKTIONEN DER STURM-LIOUVILLE-GLEICHUNG ( DIREKTES PROBLEM )      *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)
